'use strict';

/**
 * Convert config object to string with xml
 * @param config
 * @returns {string}
 */
function makeConfigString(config) {
  var serverName = config.serverName; // eslint-disable-line prefer-destructuring
  var autoUpload = config.autoUpload || false;
  var autoUploadExternalChanges = config.autoUploadExternalChanges || false;
  var notifyRemoteChanges = config.notifyRemoteChanges || true;

  var excludedLocal = [];
  var excludedServer = [];
  var mappings = [];

  if (config.excludedLocal) {
    config.excludedLocal.forEach(function (item) {
      excludedLocal.push('<excludedPath local="true" path="' + item + '" />');
    });
  }

  if (config.excludedServer) {
    config.excludedServer.forEach(function (item) {
      excludedServer.push('<excludedPath path="' + item + '" />');
    });
  }

  if (config.mappings) {
    config.mappings.forEach(function (item) {
      mappings.push('<mapping deploy="' + item.deploy + '" local="' + item.local + '" web="' + (item.web || '') + '" />');
    });
  }

  // 0.1.1 deprecation warning
  if (config.serverPath && config.localPath) {
    console.warn('Options "serverPath" and "localPath" is deprecated! Use mappings options.');
    mappings.push('<mapping deploy="' + config.serverPath + '" local="' + config.localPath + '" web="" />');
  }

  if (!serverName) {
    console.warn('Option "serverName" is not specified.');
  }

  if (mappings.length === 0) {
    console.warn('Mappings are not specified.');
  }

  return '<?xml version="1.0" encoding="UTF-8"?>\n    <project version="4">\n      <component name="PublishConfigData" promptOnRemoteOverwrite="CHECK_CONTENT" serverName="' + serverName + '" autoUpload="' + autoUpload + '" autoUploadExternalChanges="' + autoUploadExternalChanges + '" notifyRemoteChanges="' + notifyRemoteChanges + '">\n        <serverData>\n          <paths name="' + serverName + '">\n            <serverdata>\n              <mappings>\n                 ' + mappings.join('\n') + '\n              </mappings>\n              <excludedPaths>\n                ' + excludedLocal.join('\n') + '\n                ' + excludedServer.join('\n') + '\n              </excludedPaths>\n            </serverdata>\n          </paths>\n        </serverData>\n      </component>\n    </project>\n  ';
}

/**
 * Run
 * idea-deploy-config env [config]
 */
function cli() {
  var fs = require('fs');
  var args = process.argv.slice(2);
  var configSrc = args[1] || process.cwd() + '/deploy-config.json';
  var env = args[0];

  if (env === 'init') {
    console.log('Create empty deployment configuration file');
    fs.createReadStream(__dirname + '/deploy-config-example.json').pipe(fs.createWriteStream(process.cwd() + '/deploy-config.json'));
    return;
  }

  fs.readFile(configSrc, 'utf8', function (err, data) {
    if (err) throw err;

    var config = JSON.parse(data);

    if (!env) {
      env = Object.keys(config)[0]; // eslint-disable-line prefer-destructuring
      console.log('No environment set, use "' + env + '"');
      config = config[env];
    } else {
      config = config[env];
    }

    if (!config) {
      throw new Error('Configuration for ' + env + ' could not be found');
    }

    var configString = makeConfigString(config);
    var configFolder = process.cwd() + '/.idea';
    var configFile = configFolder + '/deployment.xml';

    fs.stat(configFolder, function (err) {
      if (err) throw err;

      fs.writeFile(configFile, configString, { flag: 'w' }, function (err) {
        if (err) throw err;
        console.log('IDE deployment configuration set for "' + env + '"');
      });
    });
  });
}

exports.cli = cli;